<?php

namespace Opencart\Catalog\Controller\Extension\PayAnyWay\Payment;

class Callback extends \Opencart\System\Engine\Controller{
    public function index(){
        $this->load->model('checkout/order');
        $this->load->model('account/order');

        $MNT_COMMAND = $this->getVar('MNT_COMMAND');
        $MNT_ID = $this->getVar('MNT_ID');
        $MNT_TRANSACTION_ID = $this->getVar('MNT_TRANSACTION_ID');
        $MNT_DATAINTEGRITY_CODE = $this->config->get('payment_payanyway_secret');

        $orderId = explode('-', $this->getVar('MNT_TRANSACTION_ID'));
        if(isset($orderId[0])){
            $orderId = $orderId[0];
        }
        if(empty($orderId)){
            echo 'FAIL#1';
            die;
        }
        
        //Информация о покупателе, способе доставки и платежном способе
        $orderInfo = $this->model_checkout_order->getOrder($orderId);

        //Получаем товары из корзины
        $products = $this->model_account_order->getProducts($orderId);

        if($MNT_COMMAND == 'CHECK'){
            /**
             * Ответ на уведомление по CheckURL
             */            
            if($this->checkSignatureRequest($_REQUEST, $MNT_DATAINTEGRITY_CODE)){
                $MNT_RESULT_CODE = 402;
                
                //Проверяем статус заказа. Если он отменен, аннулирован или оплачен, то отдаем MNT_RESULT_CODE 500
                $statusIds = array(5, 7, 9, 16);
                foreach($statusIds as $statusId){
                    if($orderInfo['order_status_id'] == $statusId){
                        $MNT_RESULT_CODE = 500;
                    }
                }
                
                $MNT_SIGNATURE = md5($MNT_RESULT_CODE . $MNT_ID . $MNT_TRANSACTION_ID . $MNT_DATAINTEGRITY_CODE);
                
                //Заполняем ответ номенклатурой
                $inventoryPositions = array();
                foreach($products as $product){
                    $inventoryPositions[] = array(
                        'name' => $product['name'],
                        'price' => number_format($product['total'], 2, '.', ''),
                        'quantity' => $product['quantity'],
                        'vatTag' => $this->config->get('payment_payanyway_vat'),
                    );
                }

                //Если имеется доставка и она дороже 0.0 руб, то включаем ее в номенклатуру
                if(isset($orderInfo['shipping_method']) && $orderInfo['shipping_method']['cost'] > 0.0){
                    $deliveryPositions = array(
                        'name' => $orderInfo['shipping_method']['name'],
                        'price' => number_format($orderInfo['shipping_method']['cost'], 2, '.', ''),
                        'quantity' => '1',
                        'vatTag' => 'none',
                    );

                    array_push($inventoryPositions, $deliveryPositions);
                }

                $checkResponseArray = [
                    'id' => $MNT_ID,
                    'transactionId' => $MNT_TRANSACTION_ID,
                    'amount'=> number_format($orderInfo['total'], 2, '.', ''),
                    'signature' => $MNT_SIGNATURE,
                    'resultCode' => $MNT_RESULT_CODE,
                    'externalId' => $MNT_TRANSACTION_ID,
                    'receipt' => [
                        'client' => [
                            'email' => $orderInfo['email'],
                        ],
                        'items' => $inventoryPositions,
                    ],
                ];
                
                header('Content-Type: application/json; charset=utf-8');
                echo json_encode($checkResponseArray, JSON_UNESCAPED_UNICODE);
                die;
            }else{
                echo 'FAIL#2';
                die;
            }
        }else{
            /**
             * Ответ на уведомление по PayURL
             */
            $MNT_OPERATION_ID = $this->getVar('MNT_OPERATION_ID');
            $MNT_AMOUNT = $this->getVar('MNT_AMOUNT');
            $MNT_SUBSCRIBER_ID = $this->getVar('MNT_SUBSCRIBER_ID');
            $MNT_CURRENCY_CODE = $this->getVar('MNT_CURRENCY_CODE');
            $MNT_TEST_MODE = $this->getVar('MNT_TEST_MODE');
            $MNT_SIGNATURE = $this->getVar('MNT_SIGNATURE');

            if($MNT_TRANSACTION_ID && $MNT_SIGNATURE){
                $shopSignature = md5($MNT_ID . $MNT_TRANSACTION_ID . $MNT_OPERATION_ID . $MNT_AMOUNT . $MNT_CURRENCY_CODE . $MNT_SUBSCRIBER_ID . $MNT_TEST_MODE . $MNT_DATAINTEGRITY_CODE);

                //Сверяем сигнатуру магазина с сигнатурой Монеты (PayURL)
                if($MNT_SIGNATURE == $shopSignature){
                    if($orderInfo){
                        //Добавляем заказ в БД
                        $this->model_checkout_order->addHistory($orderId, $this->config->get('payment_payanyway_order_status_id'), 'Payment Complete');

                        //Номенклатура
                        $inventoryPositions = array();
                        foreach($products as $product){
                            $inventoryPositions[] = array(
                                'name' => $product['name'],
                                'price' => number_format($product['total'], 2, '.', ''),
                                'quantity' => $product['quantity'],
                                'vatTag' => $this->getVatForCheckRequest($this->config->get('payment_payanyway_vat')),
                            );
                        }             

                        //Добавляем доставку, если она имеется
                        $deliveryPositions = null;
                        if(isset($orderInfo['shipping_method']) && $orderInfo['shipping_method']['cost'] > 0.0){
                            $deliveryPositions = number_format($orderInfo['shipping_method']['cost'], 2, '.', '');
                        }

                        $kassaInventory = json_encode($inventoryPositions);

                        $this->responseToPayURL($MNT_ID, $MNT_TRANSACTION_ID, $MNT_DATAINTEGRITY_CODE, $kassaInventory, $orderInfo['email'], $deliveryPositions);

                    }else{
                        echo 'FAIL#3';
                        die;
                    }
                }else{
                    echo 'FAIL#4';
                    die;
                }
            }else{
                echo 'FAIL#5';
                die;
            }

        }
    }

    private function getVar($name){
        $value = false;
        if(isset($_POST[$name])){
            $value = $_POST[$name];
        }elseif(isset($_GET[$name])){
            $value = $_GET[$name];
        }
        return $value;
    }

    //Режим НДС (PayURL)
    private function getVatForCheckRequest($vat){
        switch($vat){
            case 'none':
                $vat = '1105';
                break;
            case 'vat0':
                $vat = '1104';
                break;
            case 'vat10':
                $vat = '1103';
                break;
            case 'vat110':
                $vat = '1107';
                break;
            case 'vat20':
                $vat = '7000';
                break;
            case 'vat120':
                $vat = '7001';    
                break;                
        }
        return $vat;
    }    

    //Функция для сверки сигнатуры магазина с сигнатурой Монеты (CheckURL)
    private function checkSignatureRequest($data, $dataIntegrityCode){
        if(isset($data['MNT_COMMAND'], $data['MNT_ID'], $data['MNT_TRANSACTION_ID'], $data['MNT_CURRENCY_CODE'], $data['MNT_TEST_MODE'], $data['MNT_SIGNATURE'])){
            if(!isset($data['MNT_SUBSCRIBER_ID'])){
                $data['MNT_SUBSCRIBER_ID'] = '';
            }
            if(!isset($data['MNT_OPERATION_ID'])){
                $data['MNT_OPERATION_ID'] = '';
            }
            if(!isset($data['MNT_AMOUNT'])){
                $data['MNT_AMOUNT'] = '';
            }

            $shopSignature = md5($data['MNT_COMMAND'] . $data['MNT_ID'] . $data['MNT_TRANSACTION_ID'] . $data['MNT_OPERATION_ID'] . $data['MNT_AMOUNT'] . $data['MNT_CURRENCY_CODE'] . $data['MNT_SUBSCRIBER_ID'] . $data['MNT_TEST_MODE'] . $dataIntegrityCode);

            if($shopSignature == $data['MNT_SIGNATURE']){
                return true;
            }
        }else{
            return false;
        }
    }

    //Ответ на платежное уведомление (PayURL)
    private function responseToPayURL($MNT_ID, $MNT_TRANSACTION_ID, $MNT_DATAINTEGRITY_CODE, $kassaInventory = null, $kassaCustomer = null, $kassaDelivery = null){
        $resultCode = '200';
        $MNT_SIGNATURE = md5($resultCode . $MNT_ID . $MNT_TRANSACTION_ID . $MNT_DATAINTEGRITY_CODE);

        $response        = '<?xml version="1.0" encoding="UTF-8" ?>' . "\n";
        $response       .= "<MNT_RESPONSE>\n";
        $response       .= "    <MNT_ID>$MNT_ID</MNT_ID>\n";
        $response       .= "    <MNT_TRANSACTION_ID>$MNT_TRANSACTION_ID</MNT_TRANSACTION_ID>\n";
        $response       .= "    <MNT_RESULT_CODE>$resultCode</MNT_RESULT_CODE>\n";
        $response       .= "    <MNT_SIGNATURE>$MNT_SIGNATURE</MNT_SIGNATURE>\n";
        $response       .= "    <MNT_ATTRIBUTES>\n";

        if(!empty($kassaInventory) || !empty($kassaCustomer) || !empty($kassaCustomer)){
            foreach(array('INVENTORY' => $kassaInventory, 'CUSTOMER' => $kassaCustomer, 'DELIVERY' => $kassaDelivery) as $key => $value){
                if(!empty($value)){
                    $response .= "      <ATTRIBUTE>\n";
                    $response .= "          <KEY>$key</KEY>\n";
                    $response .= "          <VALUE>$value</VALUE>\n";
                    $response .= "      </ATTRIBUTE>\n";
                }
            }
        }

        $response       .= "    </MNT_ATTRIBUTES>\n";
        $response       .= "</MNT_RESPONSE>\n";

        header("Content-type: application/xml");
        echo $response;
        die;
    }
}